@@ -11,13 +11,16 @@ import math
 import shutil
 import dateparser
 import pandas as pd
+import numpy as np
+from numba import typed, types, prange
 import warnings
 from datetime import datetime
 from tqdm.auto import tqdm
 from multiprocessing import Pool, cpu_count
 from typing import Dict, Generator, List, Tuple
 from .utils import get_available_genomes, get_available_chromosomes, download_chromosome_wrapper, get_genome_informations
-from .utils import multiprocessing_gaps, multiprocessing_extract_sequences
+from .utils import multiprocessing_gaps
+from .utils import extract_sequence, extract_sequences, reverse_complement
 
 
 class Genome:
@@ -65,7 +68,7 @@ class Genome:
         self,
         assembly: str,
         chromosomes: Tuple[str] = None,
-        filters: Tuple[str] = ("chru", "scaffold", "contig", "super", "chrbin", "random", "hap", "alt", "fix"),
+        filters: Tuple[str] = ("chru", "chrmt", "scaffold", "contig", "super", "chrbin", "random", "hap", "alt", "fix"),
         verbose: bool = True,
         cache_directory: str = "genomes"
     ):
@@ -139,12 +142,11 @@ class Genome:
             self._store_chromosomes()
 
         # Filtering chromosomes
-        self._chromosomes = {
-            chrom: None
-            for chrom in self._chromosomes_lenghts
-            if all(target not in chrom.lower() for target in filters) and chromosomes is None or
-            chromosomes is not None and chrom in chromosomes
-        }
+        self._chromosomes = typed.Dict.empty(
+            types.string, types.string)
+        for chrom in self._chromosomes_lenghts:
+            if all(target not in chrom.lower() for target in filters) and chromosomes is None or chromosomes is not None and chrom in chromosomes:
+                self._chromosomes[str(chrom)] = ""
 
         # If no chromosome remains after filtering,
         # for instance when the raw data are not yet mapped
@@ -157,6 +159,22 @@ class Genome:
                     assembly=self.assembly)
             )
 
+        self.NUCLEOTIDES_MAPPING = typed.Dict.empty(types.string, types.string)
+
+        for src, dst in {
+            "a": "g",
+            "A": "G",
+            "g": "a",
+            "G": "A",
+            "t": "c",
+            "T": "C",
+            "c": "t",
+            "C": "T",
+            "N": "n",
+            "n": "N",
+        }.items():
+            self.NUCLEOTIDES_MAPPING[src] = dst
+
         self._download()
         self._load()
 
@@ -458,7 +476,7 @@ class Genome:
             # and therefore considered filled.
             # We need to add this additional row.
             chromosome_lenght = self._chromosomes_lenghts[chrom]
-            if values.chromEnd.isin([chromosome_lenght]).any():
+            if not values.chromEnd.isin([chromosome_lenght]).any():
                 non_gap_values = non_gap_values.append({
                     "chrom": chrom,
                     "chromStart": values.chromEnd.max(),
@@ -470,7 +488,7 @@ class Genome:
             # it means that the initial part of the chromosome
             # is known and therefore considered filled.
             # We need to add this additional row.
-            if values.chromStart.isin([0]).any():
+            if not values.chromStart.isin([0]).any():
                 non_gap_values = non_gap_values.append({
                     "chrom": chrom,
                     "chromStart": 0,
@@ -491,33 +509,41 @@ class Genome:
         ]))
         return pd.concat(non_gap).sort_values(["chrom"]).reset_index(drop=True)
 
-    def bed_to_sequence(self, bed: pd.DataFrame, chunksize: int = 50000):
+    def extract_sequence(self, chrom: str, chromStart: int, chromEnd: int, strand: str = ".") -> str:
+        """Return genomic sequence for given coordinates.
+
+        Parameters
+        ------------------------------------
+        chrom: str,
+            Chromosome to use.
+        chromStart: int,
+            Position from where to start slicing.
+        chromEnd: int,
+            Position where to stop the slice.
+
+        Returns
+        -------------------------------------
+        Sequence of nucleotides.
+        """
+        return extract_sequence(
+            self._chromosomes,
+            chrom,
+            chromStart,
+            chromEnd,
+            strand,
+            self.NUCLEOTIDES_MAPPING
+        )
+
+    def bed_to_sequence(self, bed: pd.DataFrame) -> List[str]:
         """Return bed with an additional column containing the sequences."""
-        tasks = [
-            {
-                "bed": group[chunksize*i:chunksize*(i+1)],
-                "sequence": self[chrom]
-            }
-            for chrom, group in bed.groupby("chrom")
-            for i in range(math.ceil(len(group)/chunksize))
-        ]
-        with Pool(min(len(tasks), cpu_count())) as p:
-            sequences = pd.concat(list(tqdm(
-                p.imap(
-                    multiprocessing_extract_sequences,
-                    tasks
-                ),
-                total=len(tasks),
-                desc="Rendering sequences in {assembly}".format(
-                    assembly=self.assembly
-                ),
-                dynamic_ncols=True,
-                disable=not self._verbose,
-                leave=False
-            )))
-            p.close()
-            p.join()
-        return sequences
+        return extract_sequences(
+            self._chromosomes,
+            bed.chrom.values.astype(str),
+            bed.chromStart.values.astype(int),
+            bed.chromEnd.values.astype(int),
+            bed.strand.values.astype(str) if "strand" in bed.columns else None,
+            self.NUCLEOTIDES_MAPPING
+        )
 
     @property
     def assembly(self) -> str:
