@@ -1,9 +1,9 @@
 from .ucsc import get_available_genomes, get_available_chromosomes, get_genome_informations, download_chromosome, download_chromosome_wrapper
 from .gaps import multiprocessing_gaps
-from .extract_sequences import multiprocessing_extract_sequences
 from .tessellate_bed import tessellate_bed
 from .expand_bed_regions import expand_bed_regions
 from .wiggle_bed_regions import wiggle_bed_regions
+from .extract_sequence import extract_sequence, reverse_complement, extract_sequences
 
 __all__ = [
     "get_available_genomes",
@@ -12,8 +12,10 @@ __all__ = [
     "download_chromosome",
     "download_chromosome_wrapper",
     "multiprocessing_gaps",
-    "multiprocessing_extract_sequences",
     "tessellate_bed",
     "expand_bed_regions",
-    "wiggle_bed_regions"
+    "wiggle_bed_regions",
+    "extract_sequence",
+    "extract_sequences",
+    "reverse_complement"
 ]
